#!/bin/bash
cd $(dirname "$BASH_SOURCE")/..
source config.cfg
BATCHPATH=$(cd $(dirname "$BASH_SOURCE")/../software; pwd)

mkdir -p $TEMPPATH
mkdir -p $DESTPATH

#VIDEO DEFINITIONS
video1080p="137"; #DASH 1080p, no audio (not DASH manifest)
video720p="22"; #Best 720p, with 192k audio (not DASH manifest)
audio256k="141"; #DASH 256k audio (only available using DASH manifest)

#FUNCTIONS
function getfilename() { #function to get filename of YouTube video
	if [ "$3" -eq "0" ]; then #don't use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames --get-filename -f $2 -o "%(title)s (%(uploader)s) %(format)s.%(ext)s"
	else #use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames --youtube-include-dash-manifest --get-filename -f $2 -o "%(title)s (%(uploader)s) %(format)s.%(ext)s"
	fi
}

function downloadfile() { #function to download YouTube video
	if [ "$3" -eq "0" ]; then #don't use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames -f $2 -o "$TEMPPATH/%(title)s (%(uploader)s) %(format)s.%(ext)s"
	else #use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames --youtube-include-dash-manifest -f $2 -o "$TEMPPATH/%(title)s (%(uploader)s) %(format)s.%(ext)s"
	fi
}

#START

videofilename=$(getfilename $1 $video1080p 0) #set videofilename to 1080p DASH video's filename

if [ -n "$videofilename" ]; then #if 1080p DASH exists, download 1080p (manifest not required)
	videofilesize="1080p"
	downloadfile $1 $video1080p 0 #download 1080p DASH
else #check if 720p exists
	echo ""
	echo "1080p video not found! Trying 720p..."
	
	videofilename=$(getfilename $1 $video720p 0) #set videofilename to 720p video
	videofilesize="720p"
fi

if [ -n "$videofilename" ]; then #if 1080p DASH or 720p exists, videofilename would have been defined
	audiofilename=$(getfilename $1 $audio256k 1) #set audiofilename
	if [ -n "$audiofilename" ]; then #If 256k audio exists, download audio (using manifest)
	    downloadfile $1 $audio256k 1
		audioquality="256k"
	fi
	
	if [[ -z "$audiofilename" || "$videofilesize" == "720p" ]];then #if 256k audio or 1080p video does not exist, need to download the 720p for the 192k audio (without manifest)
		downloadfile $1 $video720p 0 #download the 720p video

		if [ -z "$audiofilename" ]; then #if 256k audio does not exist, need to define the 192k audio
			audiofilename=$(getfilename $1 $video720p 0)
			audioquality="192k"
		fi
	fi
	
	if [[ "$audioquality" == "192k" && "$videofilesize" == "720p" ]];then #720p video always have 192k audio, but if 256k audio via manifest method exists, 720p will be multiplexed with the 256k audio
		mv -i "$TEMPPATH/$videofilename" "$DESTPATH/${videofilename%.*} nochange.mp4"
	else #mutiplex the streams of the 2 files
		$BATCHPATH/ffmpeg -i "$TEMPPATH/$videofilename" -i "$TEMPPATH/$audiofilename" -c copy -map 0:v -map 1:a "$DESTPATH/${videofilename%.*} + $audioquality audio.mp4"
		rm "$TEMPPATH/$videofilename"
		rm "$TEMPPATH/$audiofilename"
	fi
else
	echo ""
	echo "HD not found!"
fi

osascript -e 'tell application "Terminal" to quit'
exit
