#!/bin/bash 
cd $(dirname "$BASH_SOURCE")/..
source config.cfg
BATCHPATH=$(cd $(dirname "$BASH_SOURCE")/../software; pwd)

mkdir -p $TEMPPATH
mkdir -p $DESTPATH

#VIDEO DEFINITIONS
video480p="135"; #DASH 480p, no audio (not DASH manifest)
video360p="18"; #Best 360p, with 128k audio (not DASH manifest)
audio256k="141"; #DASH 256k audio (only available using DASH manifest)

#FUNCTIONS
function getfilename() { #function to get filename of YouTube video
	if [ "$3" -eq "0" ]; then #don't use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames --get-filename -f $2 -o "%(title)s (%(uploader)s) %(format)s.%(ext)s"
	else #use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames --youtube-include-dash-manifest --get-filename -f $2 -o "%(title)s (%(uploader)s) %(format)s.%(ext)s"
	fi
}

function downloadfile() { #function to download YouTube video
	if [ "$3" -eq "0" ]; then #don't use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames -f $2 -o "$TEMPPATH/%(title)s (%(uploader)s) %(format)s.%(ext)s"
	else #use DASH manifest
		$BATCHPATH/youtube-dl $1 --restrict-filenames --youtube-include-dash-manifest -f $2 -o "$TEMPPATH/%(title)s (%(uploader)s) %(format)s.%(ext)s"
	fi
}

#START

videofilename=$(getfilename $1 $video480p 0) #set videofilename to 480p DASH video's filename

if [ -n "$videofilename" ]; then #if 480p DASH exists, download 480p (manifest not required)
	videofilesize="480p"
	downloadfile $1 $video480p 0 #download 480p DASH
else #check if 360p exists
	echo ""
	echo "480p video not found! Trying 360p..."
	
	videofilename=$(getfilename $1 $video360p 0) #set videofilename to 360p video
	videofilesize="360p"
fi

if [ -n "$videofilename" ]; then #if 480p DASH or 360p exists, videofilename would have been defined
	audiofilename=$(getfilename $1 $audio256k 1) #set audiofilename
	if [ -n "$audiofilename" ]; then #If 256k audio exists, download audio (using manifest)
	    downloadfile $1 $audio256k 1
		audioquality="256k"
	fi
	
	if [[ -z "$audiofilename" || "$videofilesize" == "360p" ]];then #if 256k audio or 480p video does not exist, need to download the 360p for the 128k audio
		downloadfile $1 $video360p 0 #download the 360p video

		if [ -z "$audiofilename" ]; then #if 256k audio does not exist, need to define the 128k audio
			audiofilename=$(getfilename $1 $video360p 0)
			audioquality="128k"
		fi
	fi
	
	if [[ "$audioquality" == "128k" && "$videofilesize" == "360p" ]];then #360p video always have 128k audio, but if 256k audio via manifest method exists, 360p will be multiplexed with the 256k audio
		mv -i "$TEMPPATH/$videofilename" "$DESTPATH/${videofilename%.*} nochange.mp4"
	else #mutiplex the streams of the 2 files
		$BATCHPATH/ffmpeg -i "$TEMPPATH/$videofilename" -i "$TEMPPATH/$audiofilename" -c copy -map 0:v -map 1:a "$DESTPATH/${videofilename%.*} + $audioquality audio.mp4"
		rm "$TEMPPATH/$videofilename"
		rm "$TEMPPATH/$audiofilename"
	fi
else
	echo ""
	echo "Video quality less than 360p!"
fi

osascript -e 'tell application "Terminal" to quit'
exit
